@@ -119,44 +119,44 @@ static struct button_widget button_widgets[] = {
     { GDK_9, GDK_9,          GDK_KP_9, GDK_KP_Page_Up, GDK_R9, 0 }},
 
     {KEY_A,                  "a",
-    { 0,     0 },
-    { GDK_a, 0 }},
+    { 0,     0,     0 },
+    { GDK_a, GDK_A, 0 }},
 
     {KEY_B,                  "b",
-    { 0,     0 },
-    { GDK_b, 0 }},
+    { 0,     0,     0 },
+    { GDK_b, GDK_B, 0 }},
 
     {KEY_C,                  "c",
-    { 0,     0 },
-    { GDK_c, 0 }},
+    { 0,     0,     0 },
+    { GDK_c, GDK_C, 0 }},
 
     {KEY_D,                  "d",
-    { 0,     0 },
-    { GDK_d, 0 }},
+    { 0,     0,     0 },
+    { GDK_d, GDK_D, 0 }},
 
     {KEY_E,                  "e",
-    { 0,     0 },
-    { GDK_e, 0 }},
+    { 0,     0,     0 },
+    { GDK_e, GDK_E, 0 }},
 
     {KEY_F,                  "f",
-    { 0,     0 },
-    { GDK_f, 0 }},
+    { 0,     0,     0 },
+    { GDK_f, GDK_F, 0 }},
 
     {KEY_CLEAR,              "clear_simple",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_Delete, 0 }},
+    { GDK_Delete,     0 }},
     
     {KEY_CLEAR,              "clear_advanced",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_Delete, 0 }},
+    { GDK_Delete,     0 }},
 
     {KEY_SHIFT,              "shift_left",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_less, 0 }},
+    { GDK_less,       0 }},
 
     {KEY_SHIFT,              "shift_right",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_greater, 0 }},
+    { GDK_greater,    0 }},
 
     {KEY_SET_ACCURACY,       "accuracy",
     { GDK_SHIFT_MASK, 0 },
@@ -172,15 +172,15 @@ static struct button_widget button_widgets[] = {
 
     {KEY_STORE,              "store",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_S, 0 }},
+    { GDK_S,          0 }},
 
     {KEY_RECALL,             "recall",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_R, 0 }},
+    { GDK_R,          0 }},
 
     {KEY_EXCHANGE,           "exchange",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_X, 0 }},
+    { GDK_X,          0 }},
 
     {KEY_CLEAR_ENTRY,        "clear_entry_simple",
     { GDK_CONTROL_MASK, 0,          0 },
@@ -191,15 +191,15 @@ static struct button_widget button_widgets[] = {
     { GDK_BackSpace,    GDK_Escape, 0 }},
 
     {KEY_BACKSPACE,          "backspace_simple",
-    { 0, 0 },
+    { 0,             0 },
     { GDK_BackSpace, 0 }},
 
     {KEY_BACKSPACE,          "backspace_advanced",
-    { 0, 0 },
+    { 0,             0 },
     { GDK_BackSpace, 0 }},
     
     {KEY_NUMERIC_POINT,      "numeric_point",
-    { 0,          0,              0,             0 },
+    { 0,          0,              0,             0,                0 },
     { GDK_period, GDK_KP_Decimal, GDK_KP_Delete, GDK_KP_Separator, 0 }},
 
     {KEY_CALCULATE,          "result",
@@ -239,39 +239,39 @@ static struct button_widget button_widgets[] = {
     { GDK_C,          0 }},
 
     {KEY_INTEGER,            "integer_portion",
-    { 0, 0 },
-    { GDK_i, 0 }},
+    { 0,     0,     0 },
+    { GDK_i, GDK_I, 0 }},
 
     {KEY_FRACTION,           "fractional_portion",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_colon, 0 }},
+    { GDK_colon,      0 }},
 
     {KEY_PERCENTAGE,         "percentage",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_percent, 0 }},
+    { GDK_percent,    0 }},
 
     {KEY_SQUARE,             "square",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_at, 0 }},
+    { GDK_at,         0 }},
 
     {KEY_SQUARE_ROOT,        "sqrt",
-    { 0, 0 },   
-    { GDK_s, 0 }},
+    { 0,     0,     0 },   
+    { GDK_s, GDK_S, 0 }},
 
     {KEY_RECIPROCAL,         "reciprocal",
-    { 0, 0 },
-    { GDK_r, 0 }},
+    { 0,     0,     0 },
+    { GDK_r, GDK_R, 0 }},
 
     {KEY_ABSOLUTE_VALUE,     "abs",
-    { 0, 0 },
-    { GDK_u, 0 }},
+    { 0,     0,     0 },
+    { GDK_u, GDK_U, 0 }},
 
     {KEY_MASK_16,            "mask_16",
-    { 0, 0 },        
+    { 0,                0 },
     { GDK_bracketright, 0 }},
 
     {KEY_MASK_32,            "mask_32",
-    { 0, 0 },
+    { 0,               0 },
     { GDK_bracketleft, 0 }},
 
     {KEY_MODULUS_DIVIDE,     "modulus_divide",
@@ -284,7 +284,7 @@ static struct button_widget button_widgets[] = {
 
     {KEY_E_POW_X,            "pow_e",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_braceleft, 0 }},
+    { GDK_braceleft,  0 }},
 
     {KEY_10_POW_X,           "pow_10",
     { GDK_SHIFT_MASK, 0 },
@@ -296,35 +296,35 @@ static struct button_widget button_widgets[] = {
 
     {KEY_NATURAL_LOGARITHM,  "natural_logarithm",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_N, 0 }},
+    { GDK_N,          0 }},
 
     {KEY_LOGARITHM,          "logarithm",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_G, 0 }},
+    { GDK_G,          0 }},
 
     {KEY_LOGARITHM2,         "logarithm2",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_H, 0 }},
+    { GDK_H,          0 }},
 
     {KEY_FACTORIAL,          "factorial",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_exclam, 0 }},
+    { GDK_exclam,     0 }},
 
     {KEY_RANDOM,             "random",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_question, 0 }},
+    { GDK_question,   0 }},
 
     {KEY_SIN,                "sine",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_K, 0 }},
+    { GDK_K,          0 }},
 
     {KEY_COS,                "cosine",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_J, 0 }},
+    { GDK_J,          0 }},
 
     {KEY_TAN,                "tangent",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_L, 0 }},
+    { GDK_L,          0 }},
 
     {KEY_NOT,                "not",
     { GDK_SHIFT_MASK, 0 },
@@ -332,55 +332,55 @@ static struct button_widget button_widgets[] = {
 
     {KEY_OR,                 "or",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_bar, 0 }},
+    { GDK_bar,        0 }},
 
     {KEY_AND,                "and",
     { GDK_SHIFT_MASK, 0 },
     { GDK_ampersand, 0 }},
 
     {KEY_XOR,                "xor",
-    { 0,     0 },
-    { GDK_x, 0 }},
+    { 0,     0,     0 },
+    { GDK_x, GDK_X, 0 }},
 
     {KEY_XNOR,               "xnor",
-    { 0, 0 },
-    { GDK_n, 0 }},
+    { 0,     0,     0 },
+    { GDK_n, GDK_N, 0 }},
 
     {KEY_FINC_CTRM,          "finc_compounding_term",
-    { 0, 0 },
-    { GDK_m, 0 }},
+    { 0,     0,     0 },
+    { GDK_m, GDK_M, 0 }},
 
     {KEY_FINC_DDB,           "finc_double_declining_depreciation",
     { GDK_SHIFT_MASK, 0 },
     { GDK_D,          0 }},
 
     {KEY_FINC_FV,            "finc_future_value",
-    { 0, 0 },
-    { GDK_v, 0 }},
+    { 0,     0,     0 },
+    { GDK_v, GDK_V, 0 }},
 
     {KEY_FINC_PMT,           "finc_periodic_payment",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_P, 0 }},
+    { GDK_P,          0 }},
 
     {KEY_FINC_PV,            "finc_present_value",
-    { 0, 0 },
-    { GDK_p, 0 }},
+    { 0,     0,     0 },
+    { GDK_p, GDK_P, 0 }},
 
     {KEY_FINC_RATE,          "finc_periodic_interest_rate",
     { GDK_SHIFT_MASK, 0 },
-    { GDK_T, 0 }},
+    { GDK_T,          0 }},
 
     {KEY_FINC_SLN,           "finc_straight_line_depreciation",
-    { 0, 0 },
-    { GDK_l, 0 }},
+    { 0,     0,     0 },
+    { GDK_l, GDK_L, 0 }},
 
     {KEY_FINC_SYD,           "finc_sum_of_the_years_digits_depreciation",
-    { 0, 0 },
-    { GDK_Y, 0 }},
+    { GDK_SHIFT_MASK, 0 },
+    { GDK_Y,          0 }},
 
     {KEY_FINC_TERM,          "finc_term",
-    { 0, 0 },
-    { GDK_T, 0 }},
+    { GDK_SHIFT_MASK, 0 },
+    { GDK_T,          0 }},
 };
 #define NBUTTONS (sizeof(button_widgets) / sizeof(struct button_widget))
 
@@ -452,7 +452,7 @@ struct Xobject {               /* Gtk+/Xlib graphics object. */
     GtkWidget *disp[MAXDISPMODES];     /* Numeric display mode. */
     GtkWidget *trig[MAXTRIGMODES];     /* Trigonometric mode. */
 
-    char *lnp;                    /* Localized numerical point (UTF8 format) */
+    char *shelf;                       /* PUT selection shelf contents. */   
 
     gboolean warn_change_mode;    /* Should we warn user when changing modes? */
     gboolean bitcalculating_mode;
@@ -479,17 +479,17 @@ reset_display(void)
 
     switch (v->syntax) {
         case NPA:
-            v->noparens = 0;
+            v->ltr.noparens = 0;
             MPstr_to_num("0", DEC, v->MPdisp_val);
-            show_display(v->MPdisp_val);
+            display_set_number(v->MPdisp_val);
             clear_undo_history();
             break;
 
         case EXPRS:
             e = get_state();
             MPstr_to_num("0", DEC, e->ans);
-            exp_clear();
-            show_display(e->ans);
+            display_clear(FALSE);
+            display_set_number(e->ans);
             break;
         
         default:
@@ -571,7 +571,7 @@ ui_set_accuracy(int accuracy)
     set_int_resource(R_ACCURACY, accuracy);
     
     ui_make_registers();
-    refresh_display(-1);
+    display_refresh(-1);
     
     /* Hide the manual dialog */
     gtk_widget_hide(X->spframe);
@@ -867,7 +867,7 @@ ui_set_mode(enum mode_type mode)
 
 
 void 
-ui_set_statusbar(gchar *text, const gchar *imagename)
+ui_set_statusbar(const gchar *text, const gchar *imagename)
 {
     GtkImage *image = GTK_IMAGE(X->status_image);
 
@@ -885,17 +885,17 @@ set_bit_panel(void)
 {
     int bit_str_len, i, MP1[MP_SIZE], MP2[MP_SIZE];
     int MP[MP_SIZE];
-    char *bit_str, label[MAXLINE], tmp[MAXLINE];
+    char bit_str[MAXLINE], label[MAXLINE];
 
     switch (v->syntax) {
         case NPA:
             MPstr_to_num(v->display, v->base, MP1);
             mpcmim(MP1, MP2);
-            if (mpeq(MP1, MP2)) {
+            if (mp_is_equal(MP1, MP2)) {
                 int toclear = (v->current == KEY_CLEAR_ENTRY)
                               ? TRUE : FALSE;
 
-                bit_str = make_fixed(MP1, tmp, BIN, MAXLINE, toclear);
+                make_fixed(bit_str, MAXLINE, MP1, BIN, MAXLINE, toclear);
                 bit_str_len = strlen(bit_str);
                 if (bit_str_len <= MAXBITS) {
                     gtk_widget_set_sensitive(X->bit_panel, TRUE);
@@ -916,11 +916,11 @@ set_bit_panel(void)
             break;
 
         case EXPRS: 
-            if (usable_num(MP) || !is_integer(MP)) {
+            if (display_is_usable_number(MP) || !is_integer(MP)) {
                 gtk_widget_set_sensitive(X->bit_panel, FALSE);
                 return;
             }
-            bit_str = make_fixed(MP, tmp, BIN, MAXLINE, FALSE);
+            make_fixed(bit_str, MAXLINE, MP, BIN, MAXLINE, FALSE);
             bit_str_len = strlen(bit_str);
             if (bit_str_len <= MAXBITS) {
                 gtk_widget_set_sensitive(X->bit_panel, TRUE);
@@ -943,6 +943,27 @@ set_bit_panel(void)
     }
 }
 
+static gboolean
+redo_display(gpointer data)
+{
+    gchar *text;
+    GtkTextIter start, end, cursor;
+    gint cursor_position;
+
+    gtk_text_buffer_get_start_iter(X->display_buffer, &start);
+    gtk_text_buffer_get_end_iter(X->display_buffer, &end);
+    text = gtk_text_buffer_get_text(X->display_buffer, &start, &end, FALSE);
+    
+    g_object_get(G_OBJECT(X->display_buffer), "cursor-position", &cursor_position, NULL);
+
+    gtk_text_buffer_set_text(X->display_buffer, text, -1);    
+    gtk_text_buffer_get_iter_at_offset(X->display_buffer, &cursor, cursor_position);
+    gtk_text_buffer_place_cursor(X->display_buffer, &cursor);
+
+    g_free(text);
+    
+    return FALSE;
+}
 
 void
 ui_set_display(char *str, int cursor)
@@ -954,8 +975,8 @@ ui_set_display(char *str, int cursor)
     if (str == NULL || str[0] == '\0') {
         str = " ";
     } else {
-        if (v->noparens == 0) {
-            localize_expression(localized, str, MAX_LOCALIZED);
+        if (v->syntax == EXPRS || (v->syntax == NPA && v->ltr.noparens == 0)) {
+            localize_expression(localized, str, MAX_LOCALIZED, &cursor);
             str = localized;
         }
     }
@@ -968,6 +989,14 @@ ui_set_display(char *str, int cursor)
         gtk_text_buffer_get_iter_at_offset(X->display_buffer, &iter, cursor);
     }
     gtk_text_buffer_place_cursor(X->display_buffer, &iter);
+    gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(X->display_item), &iter, 0.0, TRUE, 1.0, 0.0);
+    
+    /* This is a workaround for bug #524602.
+     * Basically the above code can cause the display to disappear when going from
+     * a display that is wider than the widget to one that is thinner. The following
+     * causes the display to be set twice which seems to work the second time.
+     */
+    g_idle_add(redo_display, NULL);
     
     /* Align to the right */
     if (cursor < 0) {
@@ -1042,17 +1071,6 @@ ui_beep()
 }
 
 
-char *
-ui_get_localized_numeric_point(void)
-{
-    const char *decimal_point;
-
-    decimal_point = localeconv()->decimal_point;
-
-    return (g_locale_to_utf8(decimal_point, -1, NULL, NULL, NULL));
-}
-
-
 void
 ui_set_base(enum base_type base)
 {
@@ -1237,13 +1255,11 @@ static void
 aframe_response_cb(GtkWidget *dialog, gint response_id)
 {
     char *ch;
-    int val;
 
     if (response_id == GTK_RESPONSE_OK) {
         ch = (char *) gtk_entry_get_text(GTK_ENTRY(X->aframe_ch));
-        val = ch[0];
-        mpcim(&val, v->MPdisp_val);
-        show_display(v->MPdisp_val);
+        mp_set_from_integer(ch[0], v->MPdisp_val);
+        display_set_number(v->MPdisp_val);
     }
     
     gtk_widget_hide(dialog);
@@ -1309,11 +1325,25 @@ static void do_button(int function, int arg)
 {
     switch (v->syntax) {
         case NPA:
-            process_item(&buttons[function], arg);
+            v->current = buttons[function].id;
+
+            if (v->error) {
+                /* Must press a valid key first. */
+                if (v->current != KEY_CLEAR) {
+                    return;
+                }
+                ui_set_error_state(FALSE);
+            }
+    
+            if (v->ltr.noparens > 0) {
+                do_paren();
+                return;
+            }
+
+            buttons[function].func(arg);
             set_bit_panel();
-            if (v->new_input && v->dtype == FIX) {
-                STRNCPY(v->fnum, v->display, MAX_DIGITS - 1);
-                ui_set_display(v->fnum, -1);
+            if (v->ltr.new_input && v->dtype == FIX) {
+                display_set_string(v->display);
             }
             break;
 
@@ -1458,13 +1488,14 @@ exchange_menu_cb(GtkMenuItem *menu)
 static void
 update_constants_menu(void)
 {
-    char mline[MAXLINE];
+    char mline[MAXLINE], value[MAXLINE];
     int i;
 
     for (i = 0; i < MAX_CONSTANTS; i++) {
+        make_number(value, MAXLINE, v->MPcon_vals[i], DEC, TRUE);
         SNPRINTF(mline, MAXLINE, 
                  "<span weight=\"bold\">%s_%1d:</span> %s [%s]", _("C"), i, 
-                 make_number(v->MPcon_vals[i], DEC, TRUE), 
+                 value, 
                  v->con_names[i]);
         gtk_label_set_markup_with_mnemonic(GTK_LABEL(X->constant_menu_labels[i]), mline);
     }
@@ -1574,16 +1605,18 @@ create_constants_model()
     gint i = 0;
     GtkListStore *model;
     GtkTreeIter iter;
+    char constant[MAXLINE];
 
     model = gtk_list_store_new(NUM_COLUMNS, G_TYPE_INT, G_TYPE_STRING,
                                G_TYPE_STRING, G_TYPE_BOOLEAN);   
     for (i = 0; i < MAX_CONSTANTS; i++) {
         gtk_list_store_append(model, &iter);
         
+        make_number(constant, MAXLINE, v->MPcon_vals[i], DEC, TRUE);
         gtk_list_store_set(model, &iter,
                            COLUMN_NUMBER, i,
                            COLUMN_EDITABLE, TRUE,
-                           COLUMN_VALUE, g_strdup(make_number(v->MPcon_vals[i], DEC, TRUE)),
+                           COLUMN_VALUE, g_strdup(constant),
                            COLUMN_DESCRIPTION, g_strdup(v->con_names[i]),
                            -1);
     }
@@ -1620,15 +1653,16 @@ create_functions_model()
 void
 ui_make_registers()            /* Calculate memory register frame values. */
 {
-    char *mval, key[MAXLINE];
+    char mval[MAXLINE], key[MAXLINE], value[MAXLINE];
     int n;
 
     for (n = 0; n < MAX_REGISTERS; n++) {
-        mval = make_number(v->MPmvals[n], v->base, TRUE);
+        make_number(mval, MAXLINE, v->MPmvals[n], v->base, TRUE);
         gtk_entry_set_width_chars(GTK_ENTRY(X->regs[n]), strlen(mval));
         gtk_entry_set_text(GTK_ENTRY(X->regs[n]), mval);
         SNPRINTF(key, MAXLINE, "register%d", n);
-        set_resource(key, make_number(v->MPmvals[n], DEC, TRUE));
+        make_number(value, MAXLINE, v->MPmvals[n], DEC, TRUE);
+        set_resource(key, value);
     }
 }
 
@@ -1717,7 +1751,7 @@ bit_toggle_cb(GtkWidget *event_box, GdkEventButton *event)
             MPstr_to_num(v->display, v->base, MP1);
             break;
         case EXPRS: {
-            int ret = usable_num(e->ans);
+            int ret = display_is_usable_number(e->ans);
             assert(!ret);
             mpstr(e->ans, MP1);
         }
@@ -1726,7 +1760,7 @@ bit_toggle_cb(GtkWidget *event_box, GdkEventButton *event)
             assert(FALSE);
     }
 
-    mpcmd(MP1, &number);
+    number = mp_cast_to_double(MP1);
     lval = (long long) number;
 
     if (lval & (1LL << n)) {
@@ -1740,19 +1774,19 @@ bit_toggle_cb(GtkWidget *event_box, GdkEventButton *event)
 
     switch (v->syntax) {
         case NPA:
-            mpcdm(&number, v->MPdisp_val);
-            show_display(v->MPdisp_val);
+            mp_set_from_double(number, v->MPdisp_val);
+            display_set_number(v->MPdisp_val);
             break;
         case EXPRS:
-            mpcdm(&number, e->ans);
-            exp_replace("Ans");
-            refresh_display(-1);
+            mp_set_from_double(number, e->ans);
+            display_set_string("Ans");
+            display_refresh(-1);
             break;
         default:
             assert(FALSE);
     }
 
-    v->toclear = 0;
+    v->ltr.toclear = 0;
     return (TRUE);
 }
 
@@ -1801,14 +1835,15 @@ set_menubar_tooltip(gchar *menu_name)
 static void
 update_memory_menus()
 {
-    char mstr[MAXLINE];
+    char mstr[MAXLINE], value[MAXLINE];
     int i;
 
     for (i = 0; i < MAX_REGISTERS; i++) {
+        make_number(value, MAXLINE, v->MPmvals[i], v->base, TRUE);
         SNPRINTF(mstr, MAXLINE, "<span weight=\"bold\">%s_%d:</span>    %s",
         /* translators: R is the short form of register used inter alia
         in popup menus */
-                _("R"), i, make_number(v->MPmvals[i], v->base, TRUE));
+                _("R"), i, value);
         gtk_label_set_markup_with_mnemonic(GTK_LABEL(X->memory_store_labels[i]), mstr);
         gtk_label_set_markup_with_mnemonic(GTK_LABEL(X->memory_recall_labels[i]), mstr);
         gtk_label_set_markup_with_mnemonic(GTK_LABEL(X->memory_exchange_labels[i]), mstr);
@@ -1849,13 +1884,13 @@ get_display()              /* The Copy function key has been pressed. */
         string = ui_get_display();
     }
 
-    if (v->shelf != NULL) {
-        free(v->shelf);
+    if (X->shelf != NULL) {
+        free(X->shelf);
     }
-    v->shelf = g_locale_from_utf8(string, strlen(string), NULL, NULL, NULL);
+    X->shelf = g_locale_from_utf8(string, strlen(string), NULL, NULL, NULL);
     g_free(string);
 
-    gtk_clipboard_set_text(gtk_clipboard_get(X->clipboard_atom), v->shelf, -1);
+    gtk_clipboard_set_text(gtk_clipboard_get(X->clipboard_atom), X->shelf, -1);
 }
 
 
@@ -1891,7 +1926,7 @@ check_for_localized_numeric_point(int keyval)
 
     outbuf[g_unichar_to_utf8(ch, outbuf)] = '\0';
 
-    return (strcmp(outbuf, X->lnp) == 0);
+    return (strcmp(outbuf, v->radix) == 0);
 }
 
 
@@ -2172,7 +2207,7 @@ get_proc(GtkClipboard *clipboard, const gchar *buffer, gpointer data)
         case NPA:
             ret = lr_parse((char *) text, v->MPdisp_val);
             if (!ret) {
-                show_display(v->MPdisp_val);
+                display_set_number(v->MPdisp_val);
             } else {
                 ui_set_statusbar(_("Clipboard contained malformed calculation"),
                                  "gtk-dialog-error");
@@ -2180,8 +2215,8 @@ get_proc(GtkClipboard *clipboard, const gchar *buffer, gpointer data)
             break;
     
         case EXPRS:
-            exp_insert((char *) text, get_cursor()); // FIXME: Move out of gtk.c
-            refresh_display(-1);
+            display_insert((char *) text, get_cursor()); // FIXME: Move out of gtk.c
+            display_refresh(-1);
             break;
     
         default:
@@ -2234,7 +2269,7 @@ static void
 redo_cb(GtkWidget *widget)
 {
     perform_redo();
-    refresh_display(-1);
+    display_refresh(-1);
 }
 
 
@@ -2437,16 +2472,6 @@ spframe_activate_cb(GtkWidget *spin)
 }
 
 
-/*ARGSUSED*/
-void
-trig_cb(GtkWidget *widget)
-{
-    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
-        do_trigtype((enum trig_type) g_object_get_data(G_OBJECT(widget),
-                                                       "trig_mode"));
-}
-
-
 /*ARGSUSED*/
 static void
 show_thousands_separator_cb(GtkWidget *widget)
@@ -2542,7 +2567,6 @@ create_kframe()
     CONNECT_SIGNAL(mode_radio_cb);
     CONNECT_SIGNAL(inv_cb);
     CONNECT_SIGNAL(hyp_cb);
-    CONNECT_SIGNAL(trig_cb);
     CONNECT_SIGNAL(base_cb);
     CONNECT_SIGNAL(disp_cb);
     CONNECT_SIGNAL(quit_cb);
@@ -2854,7 +2878,7 @@ ui_init(int *argc, char ***argv)
         
     gtk_init(argc, argv);
 
-    X->lnp = ui_get_localized_numeric_point();
+    X->shelf      = NULL;      /* No selection for shelf initially. */
 
     gtk_rc_get_default_files();
 
