@@ -1,7 +1,7 @@
 
 /*  $Header$
  *
- *  Copyright (C) 2004-2007 Sami Pietila
+ *  Copyright (C) 2004-2008 Sami Pietila
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 
 #include "lr_parser.h"
 #include "limits.h"
-#include "extern.h"
 #include "calctool.h"
 
 /* TODO: This file is almost identical to ce-parser. */
@@ -39,7 +38,7 @@ lr_parse_(char *expression, int result[MP_SIZE], int flags)
 
     if (strlen(expression)) {
         parser_state.i = 0;
-        parser_state.buff = gc_strdup(expression);
+        parser_state.buff = strdup(expression);
         ret = lrparse();
         free(parser_state.buff);
     }
