@@ -21,9 +21,15 @@
 #include <errno.h>
 
 #include "mpmath.h"
-#include "mp.h"
-#include "calctool.h"
-#include "display.h" /* FIXME: Needed to MPstr_to_num() */
+
+static char digits[] = "0123456789ABCDEF";
+
+static double max_fix[MAXBASES] = {
+    1.298074214e+33,    /* Binary. */
+    2.037035976e+90,    /* Octal. */
+    1.000000000e+100,   /* Decimal */
+    2.582249878e+120    /* Hexadecimal. */
+};
 
 BOOLEAN
 ibool(double x)
@@ -56,10 +62,10 @@ calc_and(int t[MP_SIZE], int s1[MP_SIZE], int s2[MP_SIZE])
 {
     double dres, dval;
 
-    mpcmd(s1, &dres);
-    mpcmd(s2, &dval);
+    dres = mp_cast_to_double(s1);
+    dval = mp_cast_to_double(s2);
     dres = setbool(ibool(dres) & ibool(dval));
-    mpcdm(&dres, t);
+    mp_set_from_double(dres, t);
 }
 
 
@@ -68,10 +74,10 @@ calc_or(int t[MP_SIZE], int s1[MP_SIZE], int s2[MP_SIZE])
 {
     double dres, dval;
 
-    mpcmd(s1, &dres);
-    mpcmd(s2, &dval);
+    dres = mp_cast_to_double(s1);
+    dval = mp_cast_to_double(s2);
     dres = setbool(ibool(dres) | ibool(dval));
-    mpcdm(&dres, t);
+    mp_set_from_double(dres, t);
 }
 
 
@@ -80,10 +86,10 @@ calc_xor(int t[MP_SIZE], int s1[MP_SIZE], int s2[MP_SIZE])
 {
     double dres, dval;
 
-    mpcmd(s1, &dres);
-    mpcmd(s2, &dval);
+    dres = mp_cast_to_double(s1);
+    dval = mp_cast_to_double(s2);
     dres = setbool(ibool(dres) ^ ibool(dval));
-    mpcdm(&dres, t);
+    mp_set_from_double(dres, t);
 }
 
 
@@ -92,52 +98,45 @@ calc_xnor(int t[MP_SIZE], int s1[MP_SIZE], int s2[MP_SIZE])
 {
     double dres, dval;
 
-    mpcmd(s1, &dres);
-    mpcmd(s2, &dval);
+    dres = mp_cast_to_double(s1);
+    dval = mp_cast_to_double(s2);
     dres = setbool(~ibool(dres) ^ ibool(dval));
-    mpcdm(&dres, t);
+    mp_set_from_double(dres, t);
 }
 
 
 void
 calc_not(int s1[MP_SIZE], int t[MP_SIZE])
 {
-    double dval;
-
-    mpcmd(s1, &dval);
+    double dval = mp_cast_to_double(s1);
+    
     dval = setbool(~ibool(dval));
-    mpcdm(&dval, t);
+    mp_set_from_double(dval, t);
 }
 
 
 void
 calc_rand(int t[MP_SIZE])
 {
-    double dval = drand48();
-
-    mpcdm(&dval, t);
+    mp_set_from_double(drand48(), t);
 }
 
 
 void
 calc_u32(int s1[MP_SIZE], int t1[MP_SIZE])
 {
-    double dval;
-
-    mpcmd(s1, &dval);
+    double dval = mp_cast_to_double(s1);
     dval = setbool(ibool(dval));
-    mpcdm(&dval, t1);
+    mp_set_from_double(dval, t1);
 }
 
 
 void
 calc_u16(int s1[MP_SIZE], int t1[MP_SIZE])
 {
-    double dval;
-
-    mpcmd(s1, &dval);
+    double dval = mp_cast_to_double(s1);
     dval = setbool(ibool(dval) & 0xffff);
-    mpcdm(&dval, t1);
+    mp_set_from_double(dval, t1);
 }
 
 
@@ -146,9 +145,8 @@ calc_inv(int s1[MP_SIZE], int t1[MP_SIZE])     /* Calculate 1/x */
 {
     int MP1[MP_SIZE];
     int MP2[MP_SIZE];
-    int i = 1;
 
-    mpcim(&i, MP1);
+    mp_set_from_integer(1, MP1);
     mpstr(s1, MP2);
     mpdiv(MP1, MP2, t1);
 }
@@ -158,9 +156,7 @@ void
 calc_tenpowx(int s1[MP_SIZE], int t1[MP_SIZE])   /* Calculate 10^x */
 {
     int MP1[MP_SIZE];
-    int i = 10;
-
-    mpcim(&i, MP1);
+    mp_set_from_integer(10, MP1);
     mppwr2(MP1, s1, t1);
 }
 
@@ -168,25 +164,22 @@ calc_tenpowx(int s1[MP_SIZE], int t1[MP_SIZE])   /* Calculate 10^x */
 void
 calc_xpowy(int MPx[MP_SIZE], int MPy[MP_SIZE], int MPres[MP_SIZE]) /* Do x^y */
 {
-    int MP0[MP_SIZE], val;
+    int MP0[MP_SIZE];
 
-    do_zero(MP0);
+    mp_set_from_integer(0, MP0);
 
     /* Check if both x and y are zero. If yes, then just return 1.
      * See gcalctool bug #451286.
      */
-    if (mpeq(MPx, MP0) && mpeq(MPy, MP0)) {
-        val = 1;
-        mpcim(&val, MPres);
+    if (mp_is_equal(MPx, MP0) && mp_is_equal(MPy, MP0)) {
+        mp_set_from_integer(1, MPres);
 
-    } else if (mplt(MPx, MP0)) {          /* Is x < 0 ? */
+    } else if (mp_is_less_than(MPx, MP0)) {          /* Is x < 0 ? */
         int MPtmp[MP_SIZE];
 
         mpcmim(MPy, MPtmp);
-        if (mpeq(MPtmp, MPy)) {   /* Is y == int(y) ? */
-            int y;
-
-            mpcmi(MPy, &y);
+        if (mp_is_equal(MPtmp, MPy)) {   /* Is y == int(y) ? */
+            int y = mp_cast_to_int(MPy);
             mppwr(MPx, &y, MPres);
         } else {        /* y != int(y). Force mppwr2 to generate an error. */
             mppwr2(MPx, MPy, MPres);
@@ -222,8 +215,10 @@ calc_modulus(int op1[MP_SIZE],
     mpmul(MP1, op2, MP2);
     mpsub(op1, MP2, result);
 
-    do_zero(MP1);
-    if ((mplt(op2, MP1) && mpgt(result, MP1)) || mplt(result, MP1)) { 
+    mp_set_from_integer(0, MP1);
+    if ((mp_is_less_than(op2, MP1)
+	 && mp_is_greater_than(result, MP1)) ||
+	mp_is_less_than(result, MP1)) { 
         mpadd(result, op2, result);
     }
 
@@ -239,21 +234,10 @@ calc_percent(int s1[MP_SIZE], int t1[MP_SIZE])
     mpmul(s1, MP1, t1);
 }
 
-void
-do_zero(int t1[MP_SIZE])
-{
-    int i = 0;
-
-    mpcim(&i, t1);
-}
-
-
 void
 do_e(int t1[MP_SIZE])
 {
-    double e = 2.71828182846;
-
-    mpcdm(&e, t1);
+    mp_set_from_double(2.71828182846, t1);
 }
 
 
@@ -266,7 +250,7 @@ mptan(int s1[MP_SIZE], int t1[MP_SIZE])
 
     mpsin(s1, MPsin);
     mpcos(s1, MPcos);
-    mpcmd(MPcos, &cval);
+    cval = mp_cast_to_double(MPcos);
     if (cval == 0.0) {
         doerr(_("Error, cannot calculate cosine"));
     }
@@ -279,19 +263,17 @@ mptan(int s1[MP_SIZE], int t1[MP_SIZE])
 static void
 to_rad(int s1[MP_SIZE], int t1[MP_SIZE])
 {
-    int i, MP1[MP_SIZE], MP2[MP_SIZE];
+    int MP1[MP_SIZE], MP2[MP_SIZE];
 
     if (v->ttype == DEG) {
         mppi(MP1);
         mpmul(s1, MP1, MP2);
-        i = 180;
-        mpcim(&i, MP1);
+        mp_set_from_integer(180, MP1);
         mpdiv(MP2, MP1, t1);
     } else if (v->ttype == GRAD) {
         mppi(MP1);
         mpmul(s1, MP1, MP2);
-        i = 200;
-        mpcim(&i, MP1);
+        mp_set_from_integer(200, MP1);
         mpdiv(MP2, MP1, t1);
     } else {
         mpstr(s1, t1);
@@ -302,13 +284,12 @@ to_rad(int s1[MP_SIZE], int t1[MP_SIZE])
 static void
 do_trig_typeconv(enum trig_type ttype, int s1[MP_SIZE], int t1[MP_SIZE])
 {
-    int i, MP1[MP_SIZE], MP2[MP_SIZE];
+    int MP1[MP_SIZE], MP2[MP_SIZE];
   
     switch (ttype) {
 
         case DEG:
-            i = 180;
-            mpcim(&i, MP1);
+            mp_set_from_integer(180, MP1);
             mpmul(s1, MP1, MP2);
             mppi(MP1);
             mpdiv(MP2, MP1, t1);
@@ -319,8 +300,7 @@ do_trig_typeconv(enum trig_type ttype, int s1[MP_SIZE], int t1[MP_SIZE])
             break;
 
         case GRAD:
-            i = 200;
-            mpcim(&i, MP1);
+            mp_set_from_integer(200, MP1);
             mpmul(s1, MP1, MP2);
             mppi(MP1);
             mpdiv(MP2, MP1, t1);
@@ -359,21 +339,19 @@ mpacos(int *MPx, int *MPretval)
     int MPn1[MP_SIZE], MPpi[MP_SIZE], MPy[MP_SIZE], val;
 
     mppi(MPpi);
-    do_zero(MP0);
-    val = 1;
-    mpcim(&val, MP1);
-    val = -1;
-    mpcim(&val, MPn1);
+    mp_set_from_integer(0, MP0);
+    mp_set_from_integer(1, MP1);
+    mp_set_from_integer(-1, MPn1);
 
-    if (mpgt(MPx, MP1) || mplt(MPx, MPn1)) {
+    if (mp_is_greater_than(MPx, MP1) || mp_is_less_than(MPx, MPn1)) {
         doerr(_("Error"));
         mpstr(MP0, MPretval);
-    } else if (mpeq(MPx, MP0)) {
+    } else if (mp_is_equal(MPx, MP0)) {
         val = 2;
         mpdivi(MPpi, &val, MPretval);
-    } else if (mpeq(MPx, MP1)) {
+    } else if (mp_is_equal(MPx, MP1)) {
         mpstr(MP0, MPretval);
-    } else if (mpeq(MPx, MPn1)) {
+    } else if (mp_is_equal(MPx, MPn1)) {
         mpstr(MPpi, MPretval);
     } else { 
         mpmul(MPx, MPx, MP2);
@@ -381,7 +359,7 @@ mpacos(int *MPx, int *MPretval)
         mpsqrt(MP2, MP2);
         mpdiv(MP2, MPx, MP2);
         mpatan(MP2, MPy);
-        if (mpgt(MPx, MP0)) {
+        if (mp_is_greater_than(MPx, MP0)) {
             mpstr(MPy, MPretval);
         } else {
             mpadd(MPy, MPpi, MPretval);
@@ -400,16 +378,15 @@ mpacos(int *MPx, int *MPretval)
 static void
 mpacosh(int *MPx, int *MPretval)
 {
-    int MP1[MP_SIZE], val;
+    int MP1[MP_SIZE];
 
-    val = 1;
-    mpcim(&val, MP1);
-    if (mplt(MPx, MP1)) {
+    mp_set_from_integer(1, MP1);
+    if (mp_is_less_than(MPx, MP1)) {
         doerr(_("Error"));
-        do_zero(MPretval);
+        mp_set_from_integer(0, MPretval);
     } else {
+        int val = -1;
         mpmul(MPx, MPx, MP1);
-        val = -1;
         mpaddi(MP1, &val, MP1);
         mpsqrt(MP1, MP1);
         mpadd(MPx, MP1, MP1);
@@ -449,15 +426,12 @@ mpatanh(int *MPx, int *MPretval)
 {
     int MP0[MP_SIZE], MP1[MP_SIZE], MP2[MP_SIZE];
     int MP3[MP_SIZE], MPn1[MP_SIZE];
-    int val;
 
-    do_zero(MP0);
-    val = 1;
-    mpcim(&val, MP1);
-    val = -1;
-    mpcim(&val, MPn1);
+    mp_set_from_integer(0, MP0);
+    mp_set_from_integer(1, MP1);
+    mp_set_from_integer(-1, MPn1);
 
-    if (mpge(MPx, MP1) || mple(MPx, MPn1)) {
+    if (mp_is_greater_equal(MPx, MP1) || mp_is_less_equal(MPx, MPn1)) {
         doerr(_("Error"));
         mpstr(MP0, MPretval);
     } else {
@@ -481,7 +455,7 @@ mplogn(int n, int *MPx, int *MPretval)
 {
     int MP1[MP_SIZE], MP2[MP_SIZE];
 
-    mpcim(&n, MP1);
+    mp_set_from_integer(n, MP1);
     mpln(MP1, MP1);
     mpln(MPx, MP2);
     mpdiv(MP2, MP1, MPretval);
@@ -534,8 +508,8 @@ calc_ddb(int t[MP_SIZE])
     int val;
     int MPbv[MP_SIZE], MP1[MP_SIZE], MP2[MP_SIZE];
 
-    do_zero(MPbv);
-    mpcmi(v->MPmvals[3], &len);
+    mp_set_from_integer(0, MPbv);
+    len = mp_cast_to_int(v->MPmvals[3]);
     for (i = 0; i < len; i++) {
         mpsub(v->MPmvals[0], MPbv, MP1);
         val = 2;
@@ -642,8 +616,7 @@ calc_rate(int t[MP_SIZE])
     int MP1[MP_SIZE], MP2[MP_SIZE], MP3[MP_SIZE], MP4[MP_SIZE];
 
     mpdiv(v->MPmvals[0], v->MPmvals[1], MP1);
-    val = 1;
-    mpcim(&val, MP2);
+    mp_set_from_integer(1, MP2);
     mpdiv(MP2, v->MPmvals[2], MP3);
     mppwr2(MP1, MP3, MP4);
     val = -1;
@@ -690,8 +663,7 @@ calc_syd(int t[MP_SIZE])
     mpaddi(MP2, &val, MP3);
     mpaddi(v->MPmvals[2], &val, MP2);
     mpmul(v->MPmvals[2], MP2, MP4);
-    val = 2;
-    mpcim(&val, MP2);
+    mp_set_from_integer(2, MP2);
     mpdiv(MP4, MP2, MP1);
     mpdiv(MP3, MP1, MP2);
     mpsub(v->MPmvals[0], v->MPmvals[1], MP1);
@@ -733,9 +705,8 @@ calc_shift(int s[MP_SIZE], int t[MP_SIZE], int times)
    * boolean means BINARY representation
    */
 
-    double dval;
     BOOLEAN temp;
-    mpcmd(s, &dval);
+    double dval = mp_cast_to_double(s);
     temp = ibool(dval);
 
     /* There is a reason to do shift like this. Reason is that
@@ -756,26 +727,25 @@ calc_shift(int s[MP_SIZE], int t[MP_SIZE], int times)
     }
 
     dval = setbool(temp);
-    mpcdm(&dval, t);
+    mp_set_from_double(dval, t);
 }
 
 
 int
 is_integer(int MPnum[MP_SIZE])
 {
-    int i = 10000;
     int MPtt[MP_SIZE], MP0[MP_SIZE], MP1[MP_SIZE];
 
     /* Multiplication and division by 10000 is used to get around a 
      * limitation to the "fix" for Sun bugtraq bug #4006391 in the 
      * mpcmim() routine in mp.c, when the exponent is less than 1.
      */
-    mpcim(&i, MPtt);
+    mp_set_from_integer(10000, MPtt);
     mpmul(MPnum, MPtt, MP0);
     mpdiv(MP0, MPtt, MP0);
     mpcmim(MP0, MP1);
 
-    return mpeq(MP0, MP1);
+    return mp_is_equal(MP0, MP1);
 }
 
 
@@ -784,10 +754,10 @@ is_natural(int MPnum[MP_SIZE])
 {    
     int MP1[MP_SIZE];
     if (!is_integer(MPnum)) {
-	return 0;
+        return 0;
     }
     mpabs(MPnum, MP1);
-    return mpeq(MPnum, MP1);
+    return mp_is_equal(MPnum, MP1);
 }
 
 void
@@ -862,3 +832,300 @@ calc_trigfunc(enum trigfunc_type type, int s1[MP_SIZE], int t1[MP_SIZE])
 
     return(0);
 }
+
+/* Convert MP number to fixed number string in the given base to the
+ * maximum number of digits specified.
+ */
+
+void
+make_fixed(char *target, int target_len, int *MPnumber, int base, int cmax, int toclear)
+{
+    char half[MAXLINE], *optr;
+    int MP1base[MP_SIZE], MP1[MP_SIZE], MP2[MP_SIZE], MPval[MP_SIZE];
+    int ndig;                   /* Total number of digits to generate. */
+    int ddig;                   /* Number of digits to left of decimal sep. */
+    int dval, i;
+ 
+    optr = target;
+    mpabs(MPnumber, MPval);
+    mp_set_from_integer(0, MP1);
+    if (mp_is_less_than(MPnumber, MP1)) {
+        *optr++ = '-';
+    }
+
+    mp_set_from_integer(basevals[base], MP1base);
+
+    mppwr(MP1base, &v->accuracy, MP1);
+    /* FIXME: string const. if MPstr_to_num can get it */
+    SNPRINTF(half, MAXLINE, "0.5");
+    MPstr_to_num(half, DEC, MP2);
+    mpdiv(MP2, MP1, MP1);
+    mpadd(MPval, MP1, MPval);
+
+    mp_set_from_integer(1, MP2);
+    if (mp_is_less_than(MPval, MP2)) {
+        ddig = 0;
+        *optr++ = '0';
+        cmax--;
+    } else {
+        for (ddig = 0; mp_is_greater_equal(MPval, MP2); ddig++) {
+            mpdiv(MPval, MP1base, MPval);
+        }
+    }
+ 
+    ndig = MIN(ddig + v->accuracy, --cmax);
+
+    while (ndig-- > 0) {
+        if (ddig-- == 0) {
+            for (i = 0; i < strlen(v->radix); i++)
+                *optr++ = v->radix[i];
+        }
+        mpmul(MPval, MP1base, MPval);
+        dval = mp_cast_to_int(MPval);
+
+        if (dval > basevals[base]-1) {
+            dval = basevals[base]-1;
+        }
+
+        *optr++ = digits[dval];
+        dval = -dval;
+        mpaddi(MPval, &dval, MPval);
+    }    
+    *optr++ = '\0';
+    if (toclear == TRUE) {
+        v->ltr.toclear = 1;
+    }
+    v->ltr.pointed = 0;
+
+    /* Strip off trailing zeroes */
+    if (!v->show_zeroes) {
+        for (i = strlen(target) - 1; i > 1 && target[i] == '0'; i--) {
+            target[i] = '\0';
+        }
+        
+        /* If no fractional part discard radix */
+        if (strlen(target) >= strlen(v->radix) && strcmp(target + strlen(target) - strlen(v->radix), v->radix) == 0) {
+            target[strlen(target) - strlen(v->radix)] = '\0';
+        }
+    }
+}
+
+
+/* Convert engineering or scientific number in the given base. */
+
+void
+make_eng_sci(char *target, int target_len, int *MPnumber, int base)
+{
+    char half[MAXLINE], fixed[MAX_DIGITS], *optr;
+    int MP1[MP_SIZE], MPatmp[MP_SIZE], MPval[MP_SIZE];
+    int MP1base[MP_SIZE], MP3base[MP_SIZE], MP10base[MP_SIZE];
+    int i, dval, len, n;
+    int MPmant[MP_SIZE];        /* Mantissa. */
+    int ddig;                   /* Number of digits in exponent. */
+    int eng = 0;                /* Set if this is an engineering number. */
+    int exp = 0;                /* Exponent */
+    
+    if (v->dtype == ENG) {
+        eng = 1;
+    }
+    optr = target;
+    mpabs(MPnumber, MPval);
+    mp_set_from_integer(0, MP1);
+    if (mp_is_less_than(MPnumber, MP1)) {
+        *optr++ = '-';
+    }
+    mpstr(MPval, MPmant);
+
+    mp_set_from_integer(basevals[base], MP1base);
+    n = 3;
+    mppwr(MP1base, &n, MP3base);
+
+    n = 10;
+    mppwr(MP1base, &n, MP10base);
+
+    mp_set_from_integer(1, MP1);
+    mpdiv(MP1, MP10base, MPatmp);
+
+    mp_set_from_integer(0, MP1);
+    if (!mp_is_equal(MPmant, MP1)) {
+        while (!eng && mp_is_greater_equal(MPmant, MP10base)) {
+            exp += 10;
+            mpmul(MPmant, MPatmp, MPmant);
+        }
+ 
+        while ((!eng &&  mp_is_greater_equal(MPmant, MP1base)) ||
+                (eng && (mp_is_greater_equal(MPmant, MP3base) || exp % 3 != 0))) {
+            exp += 1;
+            mpdiv(MPmant, MP1base, MPmant);
+        }
+ 
+        while (!eng && mp_is_less_than(MPmant, MPatmp)) {
+            exp -= 10;
+            mpmul(MPmant, MP10base, MPmant);
+        }
+ 
+        mp_set_from_integer(1, MP1);
+        while (mp_is_less_than(MPmant, MP1) || (eng && exp % 3 != 0)) {
+            exp -= 1;
+            mpmul(MPmant, MP1base, MPmant);
+        }
+    }
+ 
+    make_fixed(fixed, MAX_DIGITS, MPmant, base, MAX_DIGITS-6, TRUE);
+    len = strlen(fixed);
+    for (i = 0; i < len; i++) {
+        *optr++ = fixed[i];
+    }
+ 
+    *optr++ = 'e';
+ 
+    if (exp < 0) {
+        exp = -exp;
+        *optr++ = '-';
+    } else {
+        *optr++ = '+';
+    }
+ 
+    SNPRINTF(half, MAXLINE, "0.5");
+    MPstr_to_num(half, DEC, MP1);
+    mpaddi(MP1, &exp, MPval);
+    mp_set_from_integer(1, MP1);
+    for (ddig = 0; mp_is_greater_equal(MPval, MP1); ddig++) {
+        mpdiv(MPval, MP1base, MPval);
+    }
+ 
+    if (ddig == 0) {
+        *optr++ = '0';
+    }
+ 
+    while (ddig-- > 0) {
+        mpmul(MPval, MP1base, MPval);
+        dval = mp_cast_to_int(MPval);
+        *optr++ = digits[dval];
+        dval = -dval;
+        mpaddi(MPval, &dval, MPval);
+    }
+    *optr++    = '\0';
+    v->ltr.toclear = 1;
+    v->ltr.pointed = 0;
+}
+
+
+/* Convert MP number to character string in the given base. */
+
+void
+make_number(char *target, int target_len, int *MPnumber, int base, int ignoreError)
+{
+    double val;
+    
+/*  NOTE: make_number can currently set v->error when converting to a double.
+ *        This is to provide the same look&feel as V3 even though gcalctool
+ *        now does internal arithmetic to "infinite" precision.
+ *
+ *  XXX:  Needs to be improved. Shouldn't need to convert to a double in
+ *        order to do these tests.
+ */
+
+    double number = mp_cast_to_double(MPnumber);
+
+    val = fabs(number);
+    if (v->error && !ignoreError) {
+        STRNCPY(target, _("Error"), target_len - 1);
+        return;
+	}
+    if ((v->dtype == ENG) ||
+        (v->dtype == SCI) ||
+        (v->dtype == FIX && val != 0.0 && (val > max_fix[base]))) {
+        make_eng_sci(target, target_len, MPnumber, base);
+    } else {
+        make_fixed(target, target_len, MPnumber, base, MAX_DIGITS, TRUE);
+    }
+}
+
+
+static int
+char_val(char chr)
+{
+    if (chr >= '0' && chr <= '9') {
+        return(chr - '0');
+    } else if (chr >= 'a' && chr <= 'f') {
+        return(chr - 'a' + 10);
+    } else if (chr >= 'A' && chr <= 'F') {
+        return(chr - 'A' + 10);
+    } else {
+        return(-1);
+    }
+}
+
+
+/* Convert string into an MP number, in the given base
+ */
+
+void
+MPstr_to_num(char *str, enum base_type base, int *MPval)
+{
+    char *optr;
+    int MP1[MP_SIZE], MP2[MP_SIZE], MPbase[MP_SIZE];
+    int i, inum;
+    int exp      = 0;
+    int exp_sign = 1;
+    int negate = 0;
+
+    mp_set_from_integer(0, MPval);
+    mp_set_from_integer(basevals[(int) base], MPbase);
+
+    optr = str;
+
+    /* Remove any initial spaces or tabs. */
+    while (*optr == ' ' || *optr == '\t') {
+        optr++;
+    }
+
+    /* Check if this is a negative number. */
+    if (*optr == '-') {
+        negate = 1;
+        optr++;
+    }
+
+    while ((inum = char_val(*optr)) >= 0) {
+        mpmul(MPval, MPbase, MPval);
+        mpaddi(MPval, &inum, MPval);
+        optr++;
+    }
+
+    if (*optr == '.' || *optr == *v->radix) {
+        optr++;
+        for (i = 1; (inum = char_val(*optr)) >= 0; i++) {
+            mppwr(MPbase, &i, MP1);
+            mp_set_from_integer(inum, MP2);
+            mpdiv(MP2, MP1, MP1);
+            mpadd(MPval, MP1, MPval);
+        optr++;
+        }
+    }
+
+    while (*optr == ' ') {
+        optr++;
+    }
+ 
+    if (*optr != '\0') {
+        if (*optr == '-') {
+            exp_sign = -1;
+        }
+ 
+        while ((inum = char_val(*++optr)) >= 0) {
+            exp = exp * basevals[(int) base] + inum;
+        }
+    }
+    exp *= exp_sign;
+
+    if (v->ltr.key_exp) {
+        mppwr(MPbase, &exp, MP1);
+        mpmul(MPval, MP1, MPval);
+    }
+
+    if (negate == 1) {
+        mpneg(MPval, MPval);
+    }
+}
+
